Package: polynom
Version: 1.4-1
Title: A Collection of Functions to Implement a Class for Univariate
        Polynomial Manipulations
Authors@R: c(person("Bill", "Venables", role = c("aut", "cre"),
                    email = "Bill.Venables@gmail.com",
                    comment = "S original"),
             person("Kurt", "Hornik", role = "aut",
                    email = "Kurt.Hornik@R-project.org",
                    comment = "R port"),
             person("Martin", "Maechler", role = "aut",
                    email = "maechler@stat.math.ethz.ch",
                    comment = "R port"))
Description: A collection of functions to implement a class for univariate
  polynomial manipulations.
Imports: stats, graphics
License: GPL-2
NeedsCompilation: no
Author: Bill Venables [aut, cre] (S original),
  Kurt Hornik [aut] (R port),
  Martin Maechler [aut] (R port)
Maintainer: Bill Venables <Bill.Venables@gmail.com>
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Packaged: 2022-04-11 02:34:34 UTC; Bill
Repository: CRAN
Date/Publication: 2022-04-11 03:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 08:47:34 UTC; unix

Package: polynom
Version: 1.4-1
Title: A Collection of Functions to Implement a Class for Univariate
        Polynomial Manipulations
Authors@R: c(person("Bill", "Venables", role = c("aut", "cre"),
                    email = "Bill.Venables@gmail.com",
                    comment = "S original"),
             person("Kurt", "Hornik", role = "aut",
                    email = "Kurt.Hornik@R-project.org",
                    comment = "R port"),
             person("Martin", "Maechler", role = "aut",
                    email = "maechler@stat.math.ethz.ch",
                    comment = "R port"))
Description: A collection of functions to implement a class for univariate
  polynomial manipulations.
Imports: stats, graphics
License: GPL-2
NeedsCompilation: no
Author: Bill Venables [aut, cre] (S original),
  Kurt Hornik [aut] (R port),
  Martin Maechler [aut] (R port)
Maintainer: Bill Venables <Bill.Venables@gmail.com>
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Packaged: 2022-04-11 02:34:34 UTC; Bill
Repository: CRAN
Date/Publication: 2022-04-11 03:00:02 UTC
Built: R 4.6.0; ; 2026-04-25 15:41:17 UTC; unix

importFrom("stats", "coef", "deriv", "poly", "predict")
importFrom("graphics", "lines", "par", "plot", "points", "grid")

export("GCD", "LCM",
       "as.polylist",
       "as.polynomial",
       "change.origin",
       "integral",
       "is.polylist",
       "is.polynomial",
       "monic",
       "poly.calc",
       "poly.from.roots",
       "poly.from.values",
       "poly.from.zeros",
       "poly.orth",
       "polylist",
       "polynomial")

S3method("[", "polylist")
S3method("GCD", "polylist")
S3method("GCD", "polynomial")
S3method("LCM", "polylist")
S3method("LCM", "polynomial")
S3method("Math", "polynomial")
S3method("Ops", "polynomial")
S3method("Summary", "polylist")
S3method("Summary", "polynomial")
S3method("as.character", "polynomial")
S3method("as.function", "polynomial")
S3method("c", "polylist")
S3method("coef", "polynomial")
S3method("deriv", "polylist")
S3method("deriv", "polynomial")
S3method("integral", "polylist")
S3method("integral", "polynomial")
S3method("lines", "polynomial")
S3method("plot", "polylist")
S3method("plot", "polynomial")
S3method("points", "polynomial")
S3method("predict", "polynomial")
S3method("print", "polylist")
S3method("print", "polynomial")
S3method("print", "summary.polynomial")
S3method("rep", "polylist")
S3method("solve", "polynomial")
S3method("summary", "polynomial")
S3method("unique", "polylist")

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(collapse = TRUE,
                      comment = "",
                      fig.height = 6,
                      fig.width = 8,
                      fig.align = "center")
                      # out.width = "0.25\\textheight")
library(polynom)
setHook("plot.new",
        list(las = function() par(las = 1),
             pch = function() par(pch = 20)),
        "append")

## -----------------------------------------------------------------------------
He <- list(polynomial(1), polynomial(0:1))
x <- polynomial()
for (n in 3:6) {
  He[[n]] <- x * He[[n-1]] - (n-2) * He[[n-2]] ## R indices start from 1, not 0
}
He <- as.polylist(He)
plot(He)
plot(deriv(He))
plot(integral(He))

## -----------------------------------------------------------------------------
x <- c(0,1,2,4)
(op <- poly.orth(x))
(fop <- lapply(op, as.function))
(P <- sapply(fop, function(f) f(x)))
zapsmall(crossprod(P))     ### Verify orthonormality

## -----------------------------------------------------------------------------
(p1 <- poly.calc(1:6))
(p2 <- change.origin(p1, 3))
predict(p1, 0:7)
predict(p2, 0:7)
predict(p2, 0:7 - 3)
(p3 <- (p1 - 2 * p2)^2)         # moderate arithmetic expression.
fp3 <- as.function(p3)          # should have 1, 2, 3 as zeros
fp3(0:4)


## -----------------------------------------------------------------------------
x <- 80:89
y <- c(487, 370, 361, 313, 246, 234, 173, 128, 88, 83)

p <- poly.calc(x, y)        ## leads to catastropic numerical failure!
predict(p, x) - y

p1 <- poly.calc(x - 84, y)  ## changing origin fixes the problem
predict(p1, x - 84) - y

plot(p1, xlim = c(80, 89) - 84, xlab = "x - 84")
points(x - 84, y, col = "red", cex = 2)

#### Can we now write the polynomial in "raw" form?

p0 <- as.function(p1)(polynomial() - 84) ## attempt to change the origin back to zero 
                                         ## leads to problems again
plot(p0, xlim = c(80, 89))
points(x, y, col = "red", cex = 2)  ## major numerical errors due to finite precision

## accumulate a la Abelson and Sussman.

accumulate <- function(f, init, x, right = TRUE) {
  if (length(x) == 0) {
    return(init)
  }
  f <- match.fun(f)
  if (right) {
    f(x[[1]], Recall(f, init, x[-1], right = TRUE))
  } else {
    Recall(f, f(init, x[[1]]), x[-1], right = FALSE)
  }
}

polynomial <- function(coef = c(0, 1)) {
  a <- as.numeric(coef)
  while ((la <- length(a)) > 1 && a[la] == 0) a <- a[-la]
  structure(a, class = "polynomial")
}

as.polynomial <- function(p) {
  if (is.polynomial(p)) p else polynomial(p)
}

is.polynomial <- function(p) {
  inherits(p, "polynomial")
}

Ops.polynomial <- function(e1, e2) {
  if (missing(e2))
    return(switch(.Generic,
                  `+` = e1,
                  `-` = polynomial(NextMethod(.Generic)),
                  stop("unsupported unary operation")))
  e1 <- unclass(e1)
  e2 <- unclass(e2)
  l1 <- length(e1)
  l2 <- length(e2)
  e1.op.e2 <- switch(.Generic,
                     `+` = , `-` = {
                       e1 <- c(e1, rep.int(0, max(0, l2 - l1)))
                       e2 <- c(e2, rep.int(0, max(0, l1 - l2)))
                       NextMethod(.Generic)
                     },
                     `*` = if (l1 == 1 || l2 == 1) {
                       e1 * e2
                     } else {
                       m <- outer(e1, e2)
                       as.vector(tapply(m, row(m) + col(m), sum))
                     },
                     `/` = {
                       if (l2 == 0) stop("unsupported polynomial division")
                       if (l2 == 1) e1/e2 else {
                         p <- rev(e1)
                         q <- rev(e2)
                         r <- rep.int(0, length(p))
                         i <- 0
                         while (length(p) >= l2) {
                           i <- i + 1
                           d <- p[1]/q[1]
                           r[i] <- d
                           p[1:l2] <- p[1:l2] - d * q
                           p <- p[-1]
                         }
                         if (i == 0) 0 else r[i:1]
                       }
                     },
                     `^` = {
                       if (l2 != 1 || e2 < 0 || e2%%1 != 0) stop("unsupported polynomial power")
                       switch(as.character(e2), `0` = 1, `1` = e1, {
                         p <- q <- polynomial(e1)
                         for (i in 2:e2) p <- p * q
                         as.numeric(p)
                       })
                     },
                     `%%` = {
                       if (l2 == 1) 0 else {
                         p <- rev(e1)
                         q <- rev(e2)
                         while (length(p) >= l2) {
                           d <- p[1]/q[1]
                           p[1:l2] <- p[1:l2] - d * q
                           p <- p[-1]
                         }
                         if (length(p) == 0) 0 else rev(p)
                       }
                     },
                     `==` = return(l1 == l2 && all(e1 == e2)),
                     `!=` = return(l1 != l2 || any(e1 != e2)),
                     stop("unsupported operation on polynomials"))
  polynomial(e1.op.e2)
}

Summary.polynomial <- function(..., na.rm = FALSE) {
  ok <- switch(.Generic,
               sum = , prod = TRUE,
               FALSE)
  if (!ok)
    stop(gettextf("Generic '%s' not defined for \"%s\" objects.", .Generic, .Class))
  switch(.Generic,
         sum = accumulate("+", as.polynomial(0), polylist(...)),
         prod = accumulate("*", as.polynomial(1), polylist(...)))
}

Math.polynomial <- function(x, ...) {
  switch(.Generic,
         round = , signif = , floor = , ceiling = , trunc = polynomial(NextMethod(.Generic)),
         stop(paste(.Generic, "unsupported for polynomials")))
}

as.character.polynomial <- function(x, decreasing = FALSE, ...) {
  p <- unclass(x)
  lp <- length(p) - 1
  names(p) <- 0:lp
  p <- p[p != 0]
  if (length(p) == 0)
    return("0")
  if (decreasing)
    p <- rev(p)
  signs <- ifelse(p < 0, "- ", "+ ")
  signs[1] <- if (signs[1] == "- ")
    "-" else ""
  np <- names(p)
  p <- as.character(abs(p))
  p[p == "1" & np != "0"] <- ""
  pow <- paste("x^", np, sep = "")
  pow[np == "0"] <- ""
  pow[np == "1"] <- "x"
  stars <- rep.int("*", length(p))
  stars[p == "" | pow == ""] <- ""
  paste(signs, p, stars, pow, sep = "", collapse = " ")
}

print.polynomial <- function(x, digits = getOption("digits"), decreasing = FALSE, ...) {
  p <- as.character.polynomial(signif(x, digits = digits), decreasing = decreasing)
  pc <- nchar(p)
  ow <- max(35, getOption("width"))
  m2 <- 0
  while (m2 < pc) {
    m1 <- m2 + 1
    m2 <- min(pc, m2 + ow)
    if (m2 < pc)
      while (substring(p, m2, m2) != " " && m2 > m1 + 1) m2 <- m2 - 1
    cat(substring(p, m1, m2), "\n")
  }
  invisible(x)
}

as.function.polynomial <- function(x, ...) {
  a <- rev(coef(x))
  w <- as.name("w")
  v <- as.name("x")
  ex <- call("{", call("<-", w, 0))
  for (i in seq_along(a)) {
    ex[[i + 2]] <- call("<-", w, call("+", a[1], call("*", v, w)))
    a <- a[-1]
  }
  ex[[length(ex) + 1]] <- w
  f <- function(x) NULL
  body(f) <- ex
  f
}

poly.orth <- function(x, degree = length(unique(x)) - 1, norm = TRUE) {
  at <- attr(poly(x, degree), "coefs")
  a <- at$alpha
  N <- at$norm2
  x <- polynomial()
  p <- list(polynomial(0), polynomial(1))
  for (j in 1:degree) p[[j + 2]] <- (x - a[j]) * p[[j + 1]] - N[j + 1]/N[j] * p[[j]]
  p <- p[-1]
  if (norm) {
    sqrtN <- sqrt(N[-1])
    for (j in 1 + 0:degree) p[[j]] <- p[[j]]/sqrtN[j]
  }
  class(p) <- "polylist"
  p
}

change.origin <- function(p, o) {
  if (!is.polynomial(p))
    stop(paste("\"", deparse(substitute(p)), "\"", " is not a polynomial"))
  o <- unclass(o[1])
  r <- predict(p, o)
  m <- 1
  p <- deriv(p)
  while (p != 0) {
    r <- c(r, predict(p, o))
    m <- m + 1
    p <- polynomial(unclass(deriv(p))/m)
  }
  polynomial(r)
}

coef.polynomial <- function(object, ...) {
  as.vector(object)
}

deriv.polynomial <- function(expr, ...) {
  expr <- unclass(expr)
  if (length(expr) == 1)
    return(polynomial(0))
  expr <- expr[-1]
  polynomial(expr * seq(along = expr))
}

integral <- function(expr, ...) {
  UseMethod("integral")
}

integral.polynomial <- function(expr, limits = NULL, ...) {
  expr <- unclass(expr)
  p <- polynomial(c(0, expr/seq(along = expr)))
  if (is.null(limits))
    p else diff(predict(p, limits))
}

lines.polynomial <- function(x, len = 1000, xlim = NULL, ylim = NULL, ...) {
  p <- x  # generic/method
  if (is.null(xlim))
    xlim <- par("usr")[1:2]
  if (is.null(ylim))
    ylim <- par("usr")[3:4]
  x <- seq(xlim[1], xlim[2], len = len)
  y <- predict(p, x)
  y[y <= ylim[1] | y >= ylim[2]] <- NA
  lines(x, y, ...)
}

monic <- function(p) {
  p <- unclass(p)
  if (all(p == 0)) {
    warning("the zero polynomial has no monic form")
    return(polynomial(0))
  }
  polynomial(p/p[length(p)])
}

plot.polynomial <- function(x, xlim = 0:1, ylim = range(Px), type = "l", len = 1000,
                            ..., xlab = "x", ylab = "P(x)") {
  p <- x  # generic/method
  if (missing(xlim))
    xlim <- range(c(0, Re(unlist(summary(p)))))
  if (any(is.na(xlim))) {
    warning("summary of polynomial fails. Using nominal xlim")
    xlim <- 0:1
  }
  if (diff(xlim) == 0)
    xlim <- xlim + c(-1, 1)/2
  if (length(xlim) > 2)
    x <- xlim else {
    eps <- diff(xlim)/100
    xlim <- xlim + c(-eps, eps)
    x <- seq(xlim[1], xlim[2], len = len)
  }
  Px <- predict(p, x)
  if (!missing(ylim))
    Px[Px < ylim[1]] <- Px[Px > ylim[2]] <- NA
  plot(x, Px, type = type, xlim = xlim, ylim = ylim,
       ..., xlab = xlab, ylab = ylab)
  grid(lty = "dashed")
  lines(x, Px, type = type, ...)
}

points.polynomial <- function(x, length = 100, ...) {
  p <- x  # generic/method
  pu <- par("usr")
  x <- seq(pu[1], pu[2], len = length)
  y <- predict(p, x)
  out <- y <= pu[3] | y >= pu[4]
  y[out] <- NA
  points(x, y, ...)
}

poly.calc <- function(x, y, tol = sqrt(.Machine$double.eps), lab = dimnames(y)[[2]]) {
  if (missing(y)) {
    p <- 1
    for (xi in x) {
      p <- c(0, p) - c(xi * p, 0)
    }
    return(polynomial(p))
  }
  if (is.matrix(y)) {
    if (length(x) != nrow(y))
      stop("x and y are inconsistent in size")
    lis <- list()
    if (is.null(lab))
      lab <- paste("p", 1:(dim(y)[2]), sep = "")
    for (i in 1:dim(y)[2]) lis[[lab[i]]] <- Recall(x, y[, i], tol)
    return(structure(lis, class = "polylist"))
  }
  if (any(toss <- duplicated(x))) {
    crit <- max(tapply(y, x, function(x) diff(range(x))))
    if (crit > tol)
      warning("some duplicated x-points have inconsistent y-values")
    keep <- !toss
    y <- y[keep]
    x <- x[keep]
  }
  if ((m <- length(x)) != length(y))
    stop("x and y(x) do not match in length!")
  if (m <= 1)
    return(polynomial(y))
  r <- 0
  for (i in 1:m) r <- r + (y[i] * unclass(Recall(x[-i])))/prod(x[i] - x[-i])
  r[abs(r) < tol] <- 0
  polynomial(r)
}

poly.from.zeros <- function(...) {
  poly.calc(unlist(list(...)))
}

poly.from.roots <- poly.from.zeros

poly.from.values <- poly.calc

predict.polynomial <- function(object, newdata, ...) {
  p <- object  # generic/method
  v <- 0
  p <- rev(unclass(p))
  for (pj in p) v <- newdata * v + pj
  v
}

print.summary.polynomial <- function(x, ...) {
  cat("\n Summary information for:\n")
  print(attr(x, "originalPolynomial"))
  cat("\n Zeros:\n")
  print(x$zeros)
  cat("\n Stationary points:\n")
  print(x$stationaryPoints)
  cat("\n Points of inflexion:\n")
  print(x$inflexionPoints)
  invisible(x)
}

solve.polynomial <- function(a, b, ...) {
  if (!missing(b))
    a <- a - b
  a <- unclass(a)
  if (a[1] == 0) {
    z <- rle(a)$lengths[1]
    a <- a[-(1:z)]
    r <- rep(0, z)
  } else {
    r <- numeric(0)
  }
  switch(as.character(length(a)),
         `0` = , `1` = r,
         `2` = sort(c(r, -a[1]/a[2])),
         {
           a <- rev(unclass(a))
           a <- (a/a[1])[-1]
           M <- rbind(-a, cbind(diag(length(a) - 1), 0))
           sort(c(r, eigen(M, symmetric = FALSE, only.values = TRUE)$values))
         })
}

summary.polynomial <- function(object, ...) {
  dp <- deriv(object)
  structure(list(zeros = solve(object),
                 stationaryPoints = solve(dp),
                 inflexionPoints = solve(deriv(dp))),
            class = "summary.polynomial",
    originalPolynomial = object)
}

.is_zero_polynomial <- function(x) {
  identical(x, as.polynomial(0))
}

.degree <- function(x) {
  length(unclass(x)) - 1
}

.GCD2 <- function(x, y) {
  if (.is_zero_polynomial(y)) {
    x
  } else {
    if (.degree(y) == 0) {
      as.polynomial(1)
    } else {
      Recall(y, x%%y)
    }
  }
}

.LCM2 <- function(x, y) {
  if (.is_zero_polynomial(x) || .is_zero_polynomial(y)) {
    return(as.polynomial(0))
  }
  (x/.GCD2(x, y)) * y
}

GCD <- function(...) {
  UseMethod("GCD")
}

GCD.polynomial <- function(...) {
  args <- c.polylist(...)
  if (length(args) < 2)
    stop("Need at least two polynomials.")
  accumulate(.GCD2, args[[1]], args[-1], FALSE)
}

GCD.polylist <- GCD.polynomial

LCM <- function(...) {
  UseMethod("LCM")
}

LCM.polynomial <- function(...) {
  args <- c.polylist(...)
  if (length(args) < 2)
    stop("Need at least two polynomials.")
  accumulate(.LCM2, args[[1]], args[-1], FALSE)
}

LCM.polylist <- LCM.polynomial

polylist <- function(...) {
  .polylist_from_list(list(...))
}

.polylist_from_list <- function(x) {
  structure(lapply(x, as.polynomial), class = "polylist")
}

is.polylist <- function(x) {
  inherits(x, "polylist")
}

as.polylist <- function(x) {
  if (is.polylist(x))
    x else if (is.list(x))
    .polylist_from_list(x) else polylist(x)
}

deriv.polylist <- function(expr, ...) {
  structure(lapply(expr, deriv), class = class(expr))
}

integral.polylist <- function(expr, ...) {
  result <- lapply(expr, integral, ...)
  if (length(result) > 0 && is.polynomial(result[[1]]))
    class(result) <- class(expr)
  result
}

plot.polylist <- function(x, xlim = 0:1, ylim = range(Px), type = "l", len = 1000,
                          ..., xlab = "x", ylab = "P(x)") {
  p <- x  # generic/method
  if (missing(xlim)) {
    ## try to cover the 'interesting' region
    xlim <- range(Re(unlist(lapply(p, summary.polynomial))))
  }
  if (any(is.na(xlim))) {
    warning("summary of polynomial fails. Using nominal xlim")
    xlim <- 0:1
  }
  if (diff(xlim) == 0)
    xlim <- xlim + c(-1, 1)/2
  if (length(xlim) > 2)
    x <- xlim else {
    eps <- diff(xlim)/100
    xlim <- xlim + c(-eps, eps)
    x <- seq(xlim[1], xlim[2], len = len)
  }
  Px <- unlist(lapply(p, predict.polynomial, x))
  if (!missing(ylim))
    Px[Px < ylim[1]] <- Px[Px > ylim[2]] <- NA
  plot(cbind(x, Px), xlab = xlab, ylab = ylab, type = "n", xlim = xlim, ylim = ylim, ...)
  grid(lty = "dashed")
  for (i in seq(along = p)) lines(p[[i]], lty = i, col = i)
  invisible()
}

print.polylist <- function(x, ...) {
  cat("List of polynomials:\n")
  y <- x
  x <- unclass(x)
  NextMethod()
  invisible(y)
}

c.polylist <- function(..., recursive = FALSE) {
  .polylist_from_list(unlist(lapply(list(...), as.polylist), recursive = FALSE))
}

"[.polylist" <- function(x, i) {
  .polylist_from_list(NextMethod("["))
}

rep.polylist <- function(x, times, ...) {
  .polylist_from_list(NextMethod("rep"))
}

unique.polylist <- function(x, incomparables = FALSE, ...) {
  .polylist_from_list(NextMethod("unique"))
}

Summary.polylist <- function(..., na.rm = FALSE) {
  ok <- switch(.Generic,
               sum = , prod = TRUE,
               FALSE)
  if (!ok)
    stop(gettextf("Generic '%s' not defined for \"%s\" objects.", .Generic, .Class))
  switch(.Generic,
         sum = accumulate("+", as.polynomial(0), c(...)),
         prod = accumulate("*", as.polynomial(1), c(...)))
}

---
title: "Univariate Polynomials in R"
author: "Bill Venables"
date: "`r Sys.Date()`"
output: 
  html_document: null
  pdf_document:
    includes:
      in_header: header.tex
vignette: >
  %\VignetteIndexEntry{Univariate Polynomial Manipulation}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE,
                      comment = "",
                      fig.height = 6,
                      fig.width = 8,
                      fig.align = "center")
                      # out.width = "0.25\\textheight")
library(polynom)
setHook("plot.new",
        list(las = function() par(las = 1),
             pch = function() par(pch = 20)),
        "append")
```

## Preamble

The `polynom` package is an R collection of functions to implement a class for
univariate polynomial manipulations.  It is based on the corresponding S
package by Bill Venables `<Bill.Venables@gmail.com>`, and was
adapted to R by Kurt Hornik `<Kurt.Hornik@R-project.org>` and Martin
Maechler `<maechler@stat.math.ethz.ch>`.

This document is based on the original 'NOTES', with minor updates.

# A Univariate Polynomial Class for R

## Introduction and summary

The following started as a straightforward programming exercise in
operator overloading, but seems to be more generally useful.  The goal
is to write a polynomial class, that is a suite of facilities that allow
operations on polynomials: addition, subtraction, multiplication,
"division", remaindering, printing, plotting, and so forth, to be
conducted using the same operators and functions, and hence with the
same ease, as ordinary arithmetic, plotting, printing, and so on.

The class is limited to univariate polynomials, and so they may
therefore be uniquely defined by their numeric coefficient vector.
Coercing a polynomial to numeric yields this coefficient vector as a
numeric vector.

For reasons of simplicity it is limited to REAL polynomials; handling
polynomials with complex coefficients would be a simple extension.
Dealing with polynomials with polynomial coefficients, and hence
multivariate polynomials, would be feasible, though a major undertaking
and the result would be very slow and of rather limited usefulness and
efficiency.

## General orientation

The function `polynomial()` creates an object of class `polynomial` from a
numeric coefficient vector.  Coefficient vectors are assumed to apply to
the powers of the carrier variable in increasing order, that is, in the
*truncated power series* form, and in the same form as required by
`polyroot()`, the system function for computing zeros of polynomials.  (As
a matter or terminology, the *zeros* of the polynomial $P(x)$ are the same
as the *roots* of equation $P(x) = 0$.)

Polynomials may also be created by specifying a set of (x, y) pairs and
constructing the Lagrange interpolation polynomial that passes through
them (`poly.calc(x, y)`).  If `y` is a matrix, an interpolation polynomial
is calculated for each column and the result is a list of polynomials
(of class `polylist`).

The third way polynomials are commonly generated is via its zeros using
`poly.calc(z)`, which creates the monic polynomial of lowest degree with
the values in `z` as its zeros.

The core facility provided is the group method function
`Ops.polynomial()`, which allows arithmetic operations to be performed on
polynomial arguments using ordinary arithmetic operators.

## Notes

 1. `+`, `-` and `*` have their obvious meanings for polynomials.
 
 2. `^` is limited to non-negative integer powers.
 
 3. `/` returns the polynomial quotient.  If division is not exact the
    remainder is discarded, (but see 4.)
 
 4. `%%` returns the polynomial remainder, so that if all arguments are
    polynomials, `p1 * (p2 / p1) + p2 %% p1` is the same polynomial as `p2`,
    provided `p1` is not the zero polynomial.
 
 5. If numeric vectors are used in polynomial arithmetic they are
    coerced to polynomial, which could be a source of surprise.  In the
    case of scalars, though, the result is natural.
 
 6. Some logical operations are allowed, but not always very
    satisfactorily.  `==` and `!=` mean exact equality or not,
    respectively, however `<`, `<=`, `>`, `>=`, `!`, `|` and `&` are not
    allowed at all and cause stops in the calculation. 
 
 7. Most Math group functions are disallowed with polynomial arguments.
    The only exceptions are `ceiling`, `floor`, `round`, `trunc`, and
    `signif`.
 
 8. Summary group functions are not implemented, apart from `sum` and `prod`.
 
 9. Polynomials may be evaluated at specific x values either directly
    using `predict(p, x)`, or indirectly using `as.function(p)`, which
    creates a function to evaluate the polynomial, and then using the
    result.
 
10. The print method for polynomials can be slow and is a bit
    pretentious.  The plotting methods (`plot(p)`, `lines(p)`, `points(p)`)
    are fairly nominal, but may prove useful.

## Examples

1.  Find the Hermite polynomials up to degree 5 and plot them.
    Also plot their derivatives and integrals on separate plots. 

    The polynomials in question satisfy 
$$ 
    \begin{aligned}
    He_0(x) &= 1,\\
    He_1(x) &= x,\\
    He_n(x) &= x  He_{n-1}(x) - (n - 1)  He_{n-2}(x), \qquad n = 2, 3, \ldots
    \end{aligned}
$$

```{r}
He <- list(polynomial(1), polynomial(0:1))
x <- polynomial()
for (n in 3:6) {
  He[[n]] <- x * He[[n-1]] - (n-2) * He[[n-2]] ## R indices start from 1, not 0
}
He <- as.polylist(He)
plot(He)
plot(deriv(He))
plot(integral(He))
```


2.  Find the orthogonal polynomials on $x = (0, 1, 2, 4)$ and
    construct R functions to evaluate them at arbitrary $x$ values.

```{r}
x <- c(0,1,2,4)
(op <- poly.orth(x))
(fop <- lapply(op, as.function))
(P <- sapply(fop, function(f) f(x)))
zapsmall(crossprod(P))     ### Verify orthonormality
```

3. Miscellaneous computations using polynomial arithmetic.

```{r}
(p1 <- poly.calc(1:6))
(p2 <- change.origin(p1, 3))
predict(p1, 0:7)
predict(p2, 0:7)
predict(p2, 0:7 - 3)
(p3 <- (p1 - 2 * p2)^2)         # moderate arithmetic expression.
fp3 <- as.function(p3)          # should have 1, 2, 3 as zeros
fp3(0:4)

```

4. Polynomials can be numerically fragile. This can easily lead to surprising numerical problems.

```{r}
x <- 80:89
y <- c(487, 370, 361, 313, 246, 234, 173, 128, 88, 83)

p <- poly.calc(x, y)        ## leads to catastropic numerical failure!
predict(p, x) - y

p1 <- poly.calc(x - 84, y)  ## changing origin fixes the problem
predict(p1, x - 84) - y

plot(p1, xlim = c(80, 89) - 84, xlab = "x - 84")
points(x - 84, y, col = "red", cex = 2)

#### Can we now write the polynomial in "raw" form?

p0 <- as.function(p1)(polynomial() - 84) ## attempt to change the origin back to zero 
                                         ## leads to problems again
plot(p0, xlim = c(80, 89))
points(x, y, col = "red", cex = 2)  ## major numerical errors due to finite precision

```

\name{summary.polynomial}
\alias{summary.polynomial}
\alias{print.summary.polynomial}
\title{Summary of a Polynomial}
\description{
  Summarize a polynomial by describing its ``key'' points.
}
\usage{
\method{summary}{polynomial}(object, \dots)
}
\arguments{
  \item{object}{an object of class \code{"polynomial"}.}
  \item{\dots}{Not used by this method.}
}
\value{
  A list of class \code{"summary.polynomial"} (which has its own
  \code{print} method) containing information on zeros, stationary and
  inflexion points.
}
\details{
  This is a method for the generic function \code{\link{summary}}.
}
\examples{
p <- polynomial(6:1)
p
## 6 + 5*x + 4*x^2 + 3*x^3 + 2*x^4 + x^5 
pz <- summary(p)
pz
## [1] -1.49180+0.0000i -0.80579-1.2229i -0.80579+1.2229i
## [4]  0.55169-1.2533i  0.55169+1.2533i
## To retrieve the original polynomial from the zeros:
poly.calc(pz)
## Warning: imaginary parts discarded in coercion
## 6 + 5*x + 4*x^2 + 3*x^3 + 2*x^4 + x^5
}
\keyword{symbolmath}

\name{polylist}
\alias{polylist}
\alias{as.polylist}
\alias{is.polylist}
\alias{[.polylist}
\alias{c.polylist}
\alias{deriv.polylist}
\alias{integral.polylist}
\alias{plot.polylist}
\alias{print.polylist}
\alias{rep.polylist}
\alias{unique.polylist}
\title{Lists of Polynomials}
\description{Create and manipulate lists of polynomials.}
\usage{
polylist(\dots)
as.polylist(x)
is.polylist(x)
}
\arguments{
  \item{\dots}{\R objects, polynomials or capable of coercion to polynomials.}
  \item{x}{an \R object.}
}
\details{
  \code{polylist} takes a list of arguments, tries to convert each into
  a polynomial (see \code{\link{polynomial}}), and sets the class of the
  list to \code{"polylist"}.

  \code{as.polylist} tries to coerce its arguments to a polylist, and
  will do so for arguments which are polynomials or lists thereof.

  \code{is.polylist} tests whether its argument is a polylist.

  This class has several useful methods, such as taking derivatives
  (\code{\link{deriv}}) and antiderivatives (\code{\link{integral}}),
  printing and plotting, subscripting, computing sums and products of
  the elements, and methods for \code{\link{c}}, \code{\link{rep}}, and
  \code{\link{unique}}.
}
\examples{
## Calculate orthogonal polynomials
pl <- poly.orth(rep(1:4, 1:4), 3)
pl
plot(pl)
deriv(pl)
integral(pl)
sum(pl)
prod(pl)
unique(rep(pl, 3)[c(8, 12)])
}
\keyword{symbolmath}

\name{predict.polynomial}
\alias{predict.polynomial}
\title{Evaluate a Polynomial}
\usage{
\method{predict}{polynomial}(object, newdata, \dots)
}
\arguments{
  \item{object}{A polynomial object to be evaluated.}
  \item{newdata}{Argument at which evaluation is requested.  May be
    numeric or itself a polynomial}
  \item{\dots}{Not used by this method.}
}
\description{
  Evaluate a polynomial at a given numeric or polynomial argument.
}
\details{
  This is a method for the generic function \code{\link{predict}}.

  The polynomial is evaluated according to the Horner scheme for speed
  and numerical accuracy.
}
\value{
  Evaluated object of the same class as \code{newdata}.
}
\seealso{
  \code{\link{as.function.polynomial}}
}
\keyword{symbolmath}

\name{as.function.polynomial}
\alias{as.function.polynomial}
\title{Coerce a Polynomial to a Function}
\description{
  Takes a polynomial argument and constructs an \R function to evaluate
  it at arbitrary points.
}
\usage{
\method{as.function}{polynomial}(x, \dots)
}
\arguments{
  \item{x}{An object of class \code{"polynomial"}.}
  \item{\dots}{further arguments to be passed to or from methods.}
}
\details{
  This is a method for the generic function \code{\link{as.function}}.

  The polynomial is evaluated within the function using the Horner
  scheme.

  Note that you can use the model-oriented \code{predict} method for
  polynomials for purpose of evaluation (without explicit coercion to
  a function), see the example below.
}
\value{
  A function to evaluate the polynomial \code{p}.
}
\seealso{
  \code{\link{as.function}},
  \code{\link{predict.polynomial}}
}
\examples{
pr <- (poly.calc(-1:1) - 2 * polynomial(c(1, 2, 1)))^2
pr
## 4 + 20*x + 33*x^2 + 16*x^3 - 6*x^4 - 4*x^5 + x^6
prf <- as.function(pr)
prf
## function (x) 
## 4 + x * (20 + x * (33 + x * (16 + x * (-6 + x * (-4 + x * (1))))))
## <environment: 0x402440f0>
prf(-3:3)
##  1024 64  0 4 64 144 64
predict(pr, -3:3)
##  1024 64  0 4 64 144 64
}
\keyword{symbolmath}

\name{zSummary.polynomial}
\alias{Summary.polynomial}
\title{Summary Group Methods for Polynomials}
\description{
  Allows summary group generics to be used on polynomial arguments.
}
\usage{
\method{Summary}{polynomial}(\dots, na.rm = FALSE)
}
\arguments{
  \item{\dots}{\R objects, the first supplied of class
    \code{"polynomial"}.} 
  \item{na.rm}{logical: should missing values be removed?}
}
\details{
  For the \code{sum} and \code{prod} functions, the sum and product of
  the given polynomials, respectively.  For the other members of the
  Summary group, an error is returned.

  Note that one could \emph{order} polynomials by divisibility, and
  define \code{min} and \code{max} as the corresponding lattice meet and
  join, i.e., the greatest common divisor and the least common multiple,
  respectively.  This is currently not provided: instead, functions
  \code{\link{GCD}} and \code{\link{LCM}} should be called directly.
}
\seealso{
  \code{\link{Math.polynomial}},
  \code{\link{Ops.polynomial}}
}
\keyword{symbolmath}

\name{deriv.polynomial}
\alias{deriv.polynomial}
\title{Differentiate a Polynomial}
\description{
  Calculates the derivative of a univariate polynomial.
}
\usage{
\method{deriv}{polynomial}(expr, \dots)
}
\arguments{
  \item{expr}{an object of class \code{"polynomial"}.}
  \item{\dots}{further arguments to be passed to or from methods.}
}
\details{
  This is a method for the generic function \code{\link{deriv}}.
}
\value{
  Derivative of the polynomial.
}
\seealso{
  \code{\link{integral.polynomial}},
  \code{\link{deriv}}.
}
\examples{
pr <- poly.calc(1:5)
pr
## -120 + 274*x - 225*x^2 + 85*x^3 - 15*x^4 + x^5
deriv(pr)
##  274 - 450*x + 255*x^2 - 60*x^3 + 5*x^4
}
\keyword{symbolmath}

\name{lines.polynomial}
\alias{lines.polynomial}
\title{Lines Method for Polynomials}
\description{
  Add a polynomial to an existing plot usually as a line plot.
}
\usage{
\method{lines}{polynomial}(x, len = 1000, xlim = NULL, ylim = NULL, \dots)
}
\arguments{
  \item{x}{an object of class \code{"polynomial"}.}
  \item{len}{size of vector at which evaluations are to be made.}
  \item{xlim, ylim}{the range of x and y values with sensible defaults.}  
  \item{\dots}{additional arguments as for the \code{\link{lines}}
    generic.}
}
\details{
  This is a method for the generic function \code{\link{lines}}.
    
  Lines representing the given polynomial are added to an existing
  plot.  Values outside the current plot region are not shown.
}
\seealso{
  \code{\link{lines}},
  \code{\link{points}},
  \code{\link{points.polynomial}},
  \code{\link{plot}},
  \code{\link{plot.polynomial}}.
}
\examples{
plot  (poly.calc(-1:5))
lines (poly.calc( 2:4), lty = 2)
points(poly.calc(-2:6), pch = 4)
}
\keyword{symbolmath}

\name{solve.polynomial}
\alias{solve.polynomial}
\title{Zeros of a Polynomial}
\description{
  Find the zeros, if any, of a given polynomial.
}
\usage{
\method{solve}{polynomial}(a, b, \dots)
}
\arguments{
  \item{a}{A polynomial object for which the zeros are required.}
  \item{b}{a numeric value specifying an additional intercept.  If
    given, the zeros of \code{a - b} are found.}
  \item{\dots}{Not used by this method.}
}
\value{
  A numeric vector, generally complex, of zeros.
}
\details{
  This is a method for the generic function \code{\link{solve}}.
    
  The zeros are found as the eigenvalues of the companion matrix,
  sorted according to their real parts.
}
\seealso{
  \code{\link{polyroot}},
  \code{\link{poly.calc}},
  \code{\link{summary.polynomial}}
}
\examples{
p <- polynomial(6:1)
p
## 6 + 5*x + 4*x^2 + 3*x^3 + 2*x^4 + x^5 
pz <- solve(p)
pz
## [1] -1.49180+0.0000i -0.80579-1.2229i -0.80579+1.2229i
## [4]  0.55169-1.2533i  0.55169+1.2533i
## To retrieve the original polynomial from the zeros:
poly.calc(pz)
## Warning: imaginary parts discarded in coercion
## 6 + 5*x + 4*x^2 + 3*x^3 + 2*x^4 + x^5
}
\keyword{symbolmath}

\name{monic}
\alias{monic}
\title{Monic Polynomials}
\description{
  Convert a polynomial to monic form by dividing by the leading
  coefficient.
}
\usage{
monic(p)
}
\arguments{
  \item{p}{A polynomial.  A warning is issued if the polynomial is
    identically zero.}
}
\details{
  Similar in effect to \code{p/as.numeric(p[length(p)])} but with some
  safeguards against leading zero coefficients.
}
\value{
  A polynomial proportional to \code{p} with leading coefficient 1.
}
\seealso{
  \code{\link{change.origin}}
}
\keyword{symbolmath}

\name{polynomial}
\alias{polynomial}
\alias{as.polynomial}
\alias{is.polynomial}
\alias{as.character.polynomial}
\alias{print.polynomial}
\alias{coef.polynomial}
\title{Polynomials}
\description{
  Construct, coerce to, test for, and print polynomial objects.
}
\usage{
polynomial(coef = c(0, 1))
as.polynomial(p)
is.polynomial(p)

\method{as.character}{polynomial}(x, decreasing = FALSE, \dots)
\method{print}{polynomial}(x, digits = getOption("digits"), decreasing = FALSE, \dots)
}
\arguments{
  \item{coef}{numeric vector, giving the polynomial coefficients in
    \emph{in}creasing order.}
  \item{p}{an arbitrary \R object.}
  \item{x}{a \code{polynomial} object.}
  \item{decreasing}{a logical specifying the order of the terms;
    in increasing (default) or decreasing powers.}
  \item{digits}{the number of significant digits to use for printing.}
  \item{\dots}{potentially further arguments passed to and from other methods.}
}
\details{
  \code{polynomial} constructs a polynomial from its coefficients,
  i.e., \code{p[1:k]} specifies the polynomial
  \deqn{p_1 + p_2 x + p_3 x^2 + \dots + p_k x^{k-1}.}{p[1] + p[2]* x +
    p[3]* x^2 + ... + p[k]* x^(k-1).}
  Internally, polynomials are simply numeric coefficient vectors of
  class \code{"polynomial"}.  Several useful methods are available for
  this class, such as coercion to character (\code{as.character()}) and
  function (\code{\link{as.function.polynomial}}), extraction of
  the coefficients (\code{coef()}), printing (using \code{as.character}),
  plotting (\code{\link{plot.polynomial}}), and computing sums and
  products of arbitrarily many polynomials.

  \code{as.polynomial} tries to coerce its arguments to a polynomial.

  \code{is.polynomial} tests whether its argument is a polynomial (in
  the sense that it has class \code{"polynomial"}.
}
\examples{
polynomial(1:4)
p <- as.polynomial(c(1,0,3,0))
p
print(p, decreasing = TRUE)
stopifnot(coef(p) == c(1,0,3))

polynomial(c(2,rep(0,10),1))
}
\keyword{symbolmath}

\name{Math.polynomial}
\alias{Math.polynomial}
\title{Math Group Methods for Polynomials}
\description{
  Group method for functions in the Math group.
}
\usage{
\method{Math}{polynomial}(x, \dots)
}
\arguments{
  \item{x}{an object of class \code{"polynomial"}.}
  \item{\dots}{further arguments to be passed to or from methods, such
    as \code{digits}.}
}
\details{
  Most math group functions are disallowed with polynomial arguments.
  The only exceptions are \code{\link{ceiling}}, \code{\link{floor}},
  \code{\link{round}}, \code{\link{trunc}}, and \code{\link{signif}}
  which may be used to transform the coefficients accordingly.
}
\value{
  A polynomial with transformed coefficients.
}
\seealso{
  \code{\link{Ops.polynomial}},
  \code{\link{Summary.polynomial}}.
}
\examples{
op <- options(digits=18)
p <- poly.from.values(1:4, (2:5)^2)
## 1 + 2.00000000000001*x + x^2
p <- round(p)
## 1 + 2*x + x^2
options(op)
}
\keyword{symbolmath}

\name{poly.orth}
\alias{poly.orth}
\title{Construct Orthogonal Polynomials}
\description{
  Construct the orthogonal polynomials on a given vector, up to a
  specified degree.
}
\usage{
poly.orth(x, degree = length(unique(x)) - 1, norm = TRUE)
}
\arguments{
  \item{x}{a numeric vector of abscissae.  When evaluated at \code{x}
    the polynomials will generate an orthonormal set.}
  \item{degree}{maximum degree required.  The default is one fewer than
    the number of distinct values in \code{x}, which is maximum
    possible.}
  \item{norm}{a logical indicating whether the polynomials should be
    normalized.}
}
\value{
  A list of class \code{"polylist"} of objects of class
  \code{"polynomial"} of degree 1, 2, \dots, \code{degree}.
}
\examples{
x <- rep(1:4, 1:4)		# x with repetitions for weighting
x
## [1] 1 2 2 3 3 3 4 4 4 4
polx <- poly.orth(x, 3)		# calculate orthogonal polynomials
polx
## List of polynomials:
## [[1]]
## 0.3162278 
## 
## [[2]]
## -0.9486833 + 0.3162278*x 
## 
## [[3]]
## 2.139203 - 1.863177*x + 0.3450328*x^2 
## 
## [[4]]
## -5.831564 + 8.80369*x - 3.803194*x^2 + 0.4930066*x^3 
v <- sapply(polx, predict, x)   # orthonormal basis
round(crossprod(v), 10)	        # check orthonormality
}
\keyword{symbolmath}

\name{change.origin}
\alias{change.origin}
\title{Change Origin for a Polynomial}
\description{
  Calculate the coefficients of a polynomial relative to a new origin
  on the x axis.
}
\usage{
change.origin(p, o)
}
\arguments{
  \item{p}{an object of class \code{"polynomial"}.}
  \item{o}{a numeric scalar representing the new origin on the original
    scale.}
}
\details{
  Let \eqn{P(x) = \sum_i p_i x^i} be a given polynomial and consider
  writing \eqn{P(x) = \sum_j q_j (x - o)^j}.  This function calculates
  the coefficients \eqn{q_j} and returns the result as a polynomial.
}
\value{
  A polynomial with coefficients relative to the re-located x axis.
}
\examples{
pr <- poly.calc(1:5)
pr
## -120 + 274*x - 225*x^2 + 85*x^3 - 15*x^4 + x^5
change.origin(pr, 3)
## 4*x - 5*x^3 + x^5
}
\keyword{symbolmath}

\name{Ops.polynomial}
\alias{Ops.polynomial}
\title{Arithmetic Ops Group Methods for Polynomials}
\description{
  Allows arithmetic operators to be used for polynomial calculations,
  such as addition, multiplication, division, etc.
}
\usage{
\method{Ops}{polynomial}(e1, e2)
}
\arguments{
  \item{e1}{an object of class \code{"polynomial"}.}
  \item{e2}{an object of class \code{"polynomial"}.}
}
\value{
  A polynomial got by performing the operation on the two arguments.
}
\seealso{
  \code{\link{Math.polynomial}},
  \code{\link{Summary.polynomial}}.
}
\examples{
p <- polynomial(c(1, 2, 1))
## 1 + 2*x + x^2
r <- poly.calc(-1 : 1)
## -1*x + x^3
(r - 2 * p)^2
## 4 + 20*x + 33*x^2 + 16*x^3 - 6*x^4 - 4*x^5 + x^6
}
\keyword{symbolmath}

\name{poly.calc}
\alias{poly.calc}
\alias{poly.from.zeros}
\alias{poly.from.roots}
\alias{poly.from.values}
\title{
Calculate Polynomials from Zeros or Values
}
\description{
  Calculate either the monic polynomial with specified zeros, or the
  Lagrange interpolation polynomial through the (x,y) points.
}
\usage{
poly.calc(x, y, tol=sqrt(.Machine$double.eps), lab=dimnames(y)[[2]])
}
\arguments{
  \item{x}{numeric vector specifying either the zeros of the desired
    polynomial if this is the only non-missing argument, or the	x-values
    for Lagrange interpolation.}
  \item{y}{numeric vector or matrix specifying the y-values for the
    Lagrange interpolation polynomial.  If \code{y} is a matrix,
    \code{nrow(y)} must equal \code{length(x)}, and each column of
    \code{y} is used separately with \code{x}.}
  \item{tol}{An absolute value tolerance, below which coefficients are
    treated as zero.}
  \item{lab}{If \code{y} is a matrix, lab is used as the names vector
    for the list result.}
}
\details{
  If \code{y} is a matrix, the result is a list of polynomials using
  each column separately.

  If \code{x} only is given, repeated zeros are allowed.  If \code{x}
  and \code{y} are given, repeated values in the \code{x} vector must
  have identical \code{y} values associated with them (up to
  \code{tol}), otherwise the first y-value only is used and a warning
  is issued.
}
\value{
  Either a polynomial object, or a list of polynomials, as appropriate.
  In the latter case the object is of class \code{"polylist"}.
}
\seealso{
  \code{\link{polynomial}}
}
\examples{
poly.calc(rep(1,3))
## -1 + 3*x - 3*x^2 + x^3 
poly.calc(0:4, (0:4)^2 + 1)
## 1 + x^2 
poly.calc(0:4, cbind(0:4, (0:4)^2 + 1), lab = letters[1:2])
## List of polynomials:
## $a:
## x
##
## $b:
## 1 + x^2 
}
\keyword{symbolmath}

\name{points.polynomial}
\alias{points.polynomial}
\title{Points Method for Polynomials}
\description{
  Add a polynomial to an existing plot usually as a point plot.
}
\usage{
\method{points}{polynomial}(x, length = 100, \dots)
}
\arguments{
  \item{x}{an object of class \code{"polynomial"}.}
  \item{length}{size of x vector at which evaluations are to be made.}
  \item{\dots}{additional arguments as for the points generic.}
}
\details{
  This is a method for the generic function \code{\link{points}}.

  Points representing the given polynomial are added to an existing
  plot.  Values outside the current plot region are not shown.
}
\seealso{
  \code{\link{plot}},
  \code{\link{lines}},
  \code{\link{points}},
  \code{\link{plot.polynomial}},
  \code{\link{lines.polynomial}}.
}
\examples{
plot(poly.calc(-1:5))
lines(poly.calc(2:4), lty=2)
points(poly.calc(-2:6), pch=4)
}
\keyword{symbolmath}

\name{plot.polynomial}
\alias{plot.polynomial}
\title{Plot Method for Polynomials}
\description{
  Plots polynomials, optionally allowing the ``interesting'' region to
  be automatically determined.
}
\usage{
\method{plot}{polynomial}(x, xlim = 0:1, ylim = range(Px), type = "l",
                len = 1000, \dots, xlab = "x", ylab = "P(x)")
}
\arguments{
  \item{x}{an object of class \code{"polynomial"}.}
  \item{xlim}{the range to be encompassed by the x axis.}
  \item{ylim}{the range to be encompassed by the y axis.}
  \item{type}{as for \code{\link{plot}()}.}
  \item{len}{number of x points drawn.}
  \item{\dots}{additional arguments as for \code{\link{plot}}.}
  \item{xlab,ylab}{graphical parameters.}
}
\details{
  This is a method for the generic function \code{\link{plot}}.

  A plot of the polynomial is produced on the currently active device.
  Unless otherwise specified, the domain is chosen to enclose the real
  parts of all zeros, stationary points and zero itself.
}
\seealso{
  \code{\link{plot}},
  \code{\link{lines}},
  \code{\link{points}},
  \code{\link{lines.polynomial}},
  \code{\link{points.polynomial}}.
}
\examples{
plot(p <- poly.calc(-1:5))
}
\keyword{symbolmath}

\name{integral.polynomial}
\alias{integral}
\alias{integral.polynomial}
\title{Integrate a Polynomial}
\description{
  Find the integral of a univariate polynomial.
}
\usage{
\method{integral}{polynomial}(expr, limits = NULL, ...)
}
\arguments{
  \item{expr}{an object of class \code{"polynomial"}.}
  \item{limits}{numeric vector of length 2 giving the integration
    limits.}
  \item{\dots}{further arguments to be passed to or from methods.}
}
\value{
  If \code{limits} is not given, the integral of \code{p} from 0 to
  \sQuote{x}'.  Otherwise, the integral with the given integration
  limits.
}
\seealso{
  \code{\link{deriv.polynomial}}
}
\examples{
p <- poly.calc(1:5)
p
## -120 + 274*x - 225*x^2 + 85*x^3 - 15*x^4 + x^5
deriv(p)
## 274 - 450*x + 255*x^2 - 60*x^3 + 5*x^4
integral(deriv(p)) - 120
## -120 + 274*x - 225*x^2 + 85*x^3 - 15*x^4 + x^5
}
\keyword{symbolmath}

\name{gcd}
\alias{GCD}
\alias{GCD.polylist}
\alias{GCD.polynomial}
\alias{LCM}
\alias{LCM.polylist}
\alias{LCM.polynomial}
\title{GCD and LCM for Polynomials}
\description{
  Compute the greatest common divisor (GCD) and least common multiple
  (LCM) of a collection of polynomials and polylists.
}
\usage{
\method{GCD}{polylist}(\dots)
\method{GCD}{polynomial}(\dots)
\method{LCM}{polylist}(\dots)
\method{LCM}{polynomial}(\dots)
}
\arguments{
  \item{\dots}{a list of objects of class \code{\link{polynomial}} or
    \code{\link{polylist}}.}
}
\examples{
pl <- polylist(poly.from.roots(-1),
               poly.from.roots(c(-1, -1)),
               poly.from.roots(1))
GCD(pl)
GCD(pl[-3])
LCM(pl)
LCM(pl, pl, pl[[2]])
}
\keyword{symbolmath}
